
from . import constant

parameters = {
        'WideResNet' : { "lr": 0.1, "epochs": 200, "weight_decay": 5e-4, "batch_size": 128, "nesterov": True,  "init_backbone":True, "init_extractor":False, },
        }

backbone_config = {
        "WideResNet28-10": {"depth": 28, "width": 10, "dropout": 0., "channels": 3,}, 
        "WideResNet40-2": {"depth": 40, "width": 2, "dropout": 0., "channels": 3,}, 
        }


train = { 
        'WideResNet28-10-standard' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet28-10-cutout' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet28-10-auto_augment-cutout' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet40-2-standard' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet40-2-cutout' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet40-2-auto_augment-cutout' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },

        # Auto Augment
        'WideResNet40-2-auto_augment' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet40-2-auto_augment-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet28-10-auto_augment' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet28-10-auto_augment-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        # CutMix
        'WideResNet40-2-cutmix' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet40-2-auto_augment-cutmix' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet40-2-cutmix-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet40-2-auto_augment-cutmix-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet28-10-cutmix' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet28-10-auto_augment-cutmix' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet28-10-cutmix-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },
        'WideResNet28-10-auto_augment-cutmix-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": True, "mixup" : False,
                                     },

        # MixUp
        'WideResNet40-2-mixup' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet40-2-auto_augment-mixup' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet40-2-mixup-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet40-2-auto_augment-mixup-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet28-10-mixup' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet28-10-auto_augment-mixup' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.standard_weights_std, 
                                        "agmax" : False,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet28-10-mixup-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },
        'WideResNet28-10-auto_augment-mixup-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : True,
                                     },

        # MIMax
        'WideResNet28-10-standard-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet28-10-cutout-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet28-10-auto_augment-cutout-agmax' : {  "backbone": 'WideResNet28-10',
                                        "backbone_config": backbone_config['WideResNet28-10'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },

        'WideResNet40-2-standard-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": False, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet40-2-cutout-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": False,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },
        'WideResNet40-2-auto_augment-cutout-agmax' : {  "backbone": 'WideResNet40-2',
                                        "backbone_config": backbone_config['WideResNet40-2'],
                                        "weights_std": constant.agmax_weights_std, 
                                        "agmax" : True,
                                        "parameters" : parameters['WideResNet'],
                                        "cutout": True, "auto_augment": True,  "no_basic_augment": False, "cutmix": False, "mixup" : False, 
                                     },

        }

